"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnterExitTraverser = void 0;
const typescript_estree_1 = require("@typescript-eslint/typescript-estree");
// Modified from https://github.com/typescript-eslint/typescript-eslint/blob/835378e505f462d965ce35cc4c81f8eee1704a30/packages/typescript-estree/src/simple-traverse.ts
class EnterExitTraverser {
    constructor(options) {
        this.options = options;
    }
    isValidNode(x) {
        return x && typeof x === 'object' && typeof x.type === 'string';
    }
    getVisitorKeysForNode(node) {
        const keys = typescript_estree_1.visitorKeys[node.type];
        return keys !== null && keys !== void 0 ? keys : [];
    }
    traverse(node, parent, property) {
        var _a, _b;
        if (!this.isValidNode(node)) {
            return;
        }
        try {
            let enterResult = this.options.enter(node, parent, property);
            if (enterResult === "ignore") {
                return;
            }
            if (enterResult === "stop" || enterResult === "skipSiblings") {
                return enterResult;
            }
            const keys = this.getVisitorKeysForNode(node);
            if (keys.length < 1) {
                return;
            }
            for (const key of keys) {
                const childOrChildren = node[key];
                if (Array.isArray(childOrChildren)) {
                    for (const child of childOrChildren) {
                        let childResult = this.traverse(child, node, key);
                        if (childResult === "skipSiblings")
                            return;
                        if (childResult === "stop")
                            return "stop";
                    }
                }
                else {
                    let childResult = this.traverse(childOrChildren, node, key);
                    if (childResult === "skipSiblings")
                        return;
                    if (childResult === "stop")
                        return "stop";
                }
            }
        }
        finally {
            (_b = (_a = this.options).exit) === null || _b === void 0 ? void 0 : _b.call(_a, node, parent, property);
        }
    }
}
exports.EnterExitTraverser = EnterExitTraverser;
//# sourceMappingURL=enterExitTraverser.js.map